//+------------------------------------------------------------------+
//|                             RSI SelfAdjustRSI v1.0[FX5](mtf).mq4 |
//|                                                              FX5 |
//|                                                    hazem@uk2.net |
//+------------------------------------------------------------------+
//mtf 2007fxtsd     ki   

#property copyright "FX5"
#property link      "hazem@uk2.net"
//----
#property indicator_separate_window
#property indicator_minimum 1
#property indicator_maximum 100
#property indicator_buffers 3
#property indicator_level1  20
#property indicator_level2  30
#property indicator_level3  50
#property indicator_level4  70
#property indicator_level5  80
#property indicator_levelcolor  SlateGray

//---- input parameters

extern int    rsiPeriod = 14;
extern double diviation = 2;
extern double MA_Method = 1;
extern color  rsiColor = LimeGreen;
extern color  overBoughtColor = Blue;
extern color  overSoldColor = Red;

extern int timeFrame =0;
extern string note_TimeFrames = "M1;5,15,30,60(H1);H4;D1;W1;MN";
extern string note_MA_Method  = "0/1: MA_Method/StandardDeviationMethod";

//---- buffers
double rsi[];
double upperBorder[];
double lowerBorder[];
double absDiviation[];
string IndicatorFileName;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   IndicatorBuffers(4);
   
   SetIndexStyle(0, DRAW_LINE, EMPTY, EMPTY, overBoughtColor);
   SetIndexStyle(1, DRAW_LINE, EMPTY, EMPTY, overSoldColor);
   SetIndexStyle(2, DRAW_LINE, EMPTY, EMPTY, rsiColor);
//----   
   SetIndexBuffer(0, upperBorder);   
   SetIndexBuffer(1, lowerBorder);
   SetIndexBuffer(2, rsi);
   SetIndexBuffer(3, absDiviation);      

//----   
   SetIndexDrawBegin(2, rsiPeriod);
   SetIndexDrawBegin(0, rsiPeriod * 2);
   SetIndexDrawBegin(1, rsiPeriod * 2);
//----   
   SetIndexLabel(2, "RSI");
   SetIndexLabel(0, "OverBought");
   SetIndexLabel(1, "OverSold");   
//----

 
 
   if (timeFrame < Period()) timeFrame = Period();
   IndicatorShortName("SelfAdjustRSI (" + rsiPeriod + ", "+DoubleToStr(diviation,3)+") [" + timeFrame + "] ");
   IndicatorFileName = WindowExpertName();
 

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
  {


   int      limit,i;

   int countedBars = IndicatorCounted();
   if(countedBars < 0) return(-1);
   
   if(countedBars > 0) countedBars--;
   limit = Bars-countedBars;

   //
   //
   //    
   //
   //

   if (timeFrame != Period())
      {
         limit = MathMax(limit,timeFrame/Period());
         datetime TimeArray[];
            ArrayCopySeries(TimeArray ,MODE_TIME ,NULL,timeFrame);
         
            //
            //
            //
            //
            //
         
            for(i=0,int y=0; i<limit; i++)
            {
               if(Time[i]<TimeArray[y]) y++;
                     rsi[i]           = iCustom(NULL,timeFrame,IndicatorFileName,rsiPeriod,diviation,MA_Method,overBoughtColor,overSoldColor,rsiColor,2,y);
                     upperBorder[i]   = iCustom(NULL,timeFrame,IndicatorFileName,rsiPeriod,diviation,MA_Method,overBoughtColor,overSoldColor,rsiColor,0,y);
                     lowerBorder[i]   = iCustom(NULL,timeFrame,IndicatorFileName,rsiPeriod,diviation,MA_Method,overBoughtColor,overSoldColor,rsiColor,1,y);

            }
         return(0);         
      }
      
   //
   //
   //
   //
   //


//----   
   if(MA_Method == true)
       MovingAverageMethod(limit);
   else
       StandardDeviationMethod(limit);
//----         
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void MovingAverageMethod(int limit)
  {

   for(int i = limit; i >= 0; i--)
     {     
       rsi[i] = iRSI(NULL, 0, rsiPeriod, PRICE_CLOSE, i);
       double smoothedRSI = GetSmoothedRSI(i);
       absDiviation[i] = MathAbs(rsi[i] - smoothedRSI);
       double kDiviation = diviation * GetAbsDiviationAverage(i);
       upperBorder[i] = 50 + kDiviation;
       lowerBorder[i] = 50 - kDiviation;
     }            
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void StandardDeviationMethod(int limit)
  {
   for(int i = limit; i >= 0; i--)
     {     
       rsi[i] = iRSI(NULL, 0, rsiPeriod, PRICE_CLOSE, i);
       double rsiDiviation = iStdDev(rsi, 0, rsiPeriod, 0, MODE_SMA,0, i);
       double kDiviation = diviation * rsiDiviation;
       upperBorder[i] = 50 + kDiviation;
       lowerBorder[i] = 50 - kDiviation;      
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetSmoothedRSI(int shift)
  {
   double sum = 0;
   for(int i = shift; i < shift + rsiPeriod; i++)
     {
       sum += rsi[i];
     }
   return(sum / rsiPeriod);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetAbsDiviationAverage(int shift)
  {
   double sum = 0;
   for(int i = shift; i < shift + rsiPeriod; i++)
     {
       sum += absDiviation[i];
     }
   return(sum / rsiPeriod);
  }
//+------------------------------------------------------------------+

